module Pod
    class Podfile
        module DSL
    
            # Cocoapods have a bug: doesn't generate modulemap for pure binary pod 
            # https://code.byted.org/iOS_Library/MobileDev/issues/256
            #
            # This function could fix this bug. 
            # Module if recommanded for swift project, so that you can use import XXX directly
            # 
            def bd_fix_module_for_binary_pod!
                if Pod::VERSION.start_with? "1.4"
                    raise "bd_fix_module_for_binary_pod! doesn't support cocoapods 1.4, use 1.5 or 1.6"
                end
                tmp_path = ENV["TMPDIR"]

                if Pod::VERSION.start_with? "1.5"
                    file_path = Pathname.new(tmp_path) + "bd_fix_module_1_5.rb"
                    File.write(file_path, $BD_fix_module_code_1_5)
                    load file_path.to_s
                    File.delete file_path
                else
                    # 1.6 or above
                    file_path = Pathname.new(tmp_path) + "bd_fix_module_1_6.rb"
                    File.write(file_path, $BD_fix_module_code_1_6)
                    load file_path.to_s
                    File.delete file_path
                end
            end
        end
    end
end



$BD_fix_module_code_1_6 = %q{
# ------ fix for cocoapods 1.6 -------

#
# REASON:
#
# When a pod only have vendored library, and the 'use_modular_header' is on, cocoapods won't
# generate a modulemap (so module is not working), beacause it consider it as "should_build? == false".
#
# It's a bug of cocoapods.
#
# We fix it by changing the `install!` method:
#
#    when a pod is (target.should_build? == true && target.uses_modular_headers?),
#    we treat it differently: like nomarl pods,
#       - create_umbrella_header
#       - create_module_map
#

module Pod
  class PodTarget

    def contain_vendored_libraries_and_no_vendored_frameworks?
      return @uses_libraries_no_frameworks if defined? @uses_libraries_no_frameworks
      @uses_libraries_no_frameworks = begin
        accessors = file_accessors.reject { |a| a.spec.test_specification? }
        !accessors.empty? && accessors.all? do |file_accessor|
          !file_accessor.vendored_libraries.blank? && file_accessor.vendored_frameworks.blank?
        end
      end
    end

    def should_generate_module_map_for_a_pod_should_not_build?
      @should_generate_module_map_for_a_pod_should_not_build ||= begin
        self.uses_modular_headers? && contain_vendored_libraries_and_no_vendored_frameworks?
      end
    end

  end
end

module Pod
  class Installer
    class Xcode
      class PodsProjectGenerator
        class PodTargetInstaller


          FIX_MODULE_OLD_install = instance_method(:install!)
          define_method(:install!) do

              if (not target.should_build?) && target.should_generate_module_map_for_a_pod_should_not_build?
                  # special handling for pod with no source code

                  # Most code below are copied from original install! method
                  native_target = add_target
                  _, file_accessors = target.file_accessors.partition { |fa| fa.spec.test_specification? }
                  resource_bundle_targets = add_resources_bundle_targets(file_accessors).values.flatten
                  create_xcconfig_file(native_target, resource_bundle_targets)

                  if target.defines_module?
                    create_module_map(native_target) do |generator|
                      generator.headers.concat module_map_additional_headers
                    end
                    create_umbrella_header(native_target) do |generator|
                      generator.imports += file_accessors.flat_map do |file_accessor|
                        header_dir = if !target.requires_frameworks? && dir = file_accessor.spec_consumer.header_dir
                                       Pathname.new(dir)
                                     end

                        file_accessor.public_headers.map do |public_header|
                          public_header = if header_mappings_dir
                                            public_header.relative_path_from(header_mappings_dir)
                                          else
                                            public_header.basename
                                          end
                          if header_dir
                            public_header = header_dir.join(public_header)
                          end
                          public_header
                        end
                      end
                    end
                  end

                  if target.requires_frameworks?
                    unless skip_info_plist?(native_target)
                      create_info_plist_file(target.info_plist_path, native_target, target.version, target.platform)
                    end
                    create_build_phase_to_symlink_header_folders(native_target)
                  elsif target.uses_swift?
                    add_swift_static_library_compatibility_header_phase(native_target)
                  end


                  clean_support_files_temp_dir
                  TargetInstallationResult.new(target, native_target, resource_bundle_targets)


              else
                  # call original
                  FIX_MODULE_OLD_install.bind(self).()
              end

          end # patch ended

        end
      end
    end
  end
end



module Pod
  class Target
    # @since 1.5.0
    class BuildSettings

        class PodTargetSettings

              # Patch (because there is a shuld_build? check)
              #
              FIX_MODULE_OLD_module_map_file_to_import = instance_method(:module_map_file_to_import)
              define_method(:module_map_file_to_import) do

                if (not target.should_build?) && target.should_generate_module_map_for_a_pod_should_not_build?
                    # ---- Code bellow is copied form the original method ----
                    #
                    # ---------------- this line is deleted ------------------
                    # return unless target.should_build?
                    # -------------------------------------------------------
                    #
                    return if target.requires_frameworks?
                    return unless target.defines_module?

                    if target.uses_swift?
                      # for swift, we have a custom build phase that copies in the module map, appending the .Swift module
                      "${PODS_CONFIGURATION_BUILD_DIR}/#{target.label}/#{target.product_module_name}.modulemap"
                    else
                      "${PODS_ROOT}/#{target.module_map_path.relative_path_from(target.sandbox.root)}"
                    end
                    # ---- Code above is copied form the original method ----
                else
                    # call original
                    FIX_MODULE_OLD_module_map_file_to_import.bind(self).()
                end

              end # patch ended

        end

    end
  end
end


# ------ \fix for cocoapods 1.6 -------
}

$BD_fix_module_code_1_5 = %q{
# ------ fix for cocoapods 1.5 ------- 

#
# REASON:
#
# When a pod only have vendored library, and the 'use_modular_header' is on, cocoapods won't
# generate a modulemap (so module is not working), beacause it consider it as "should_build? == false".
#
# It's a bug of cocoapods.
#
# We fix it by changing the `install!` method:
#
#    when a pod is (target.should_build? == true && target.uses_modular_headers?),
#    we treat it differently: like nomarl pods,
#       - create_umbrella_header
#       - create_module_map

module Pod
  class PodTarget

    def contain_vendored_libraries_and_no_vendored_frameworks?
      return @uses_libraries_no_frameworks if defined? @uses_libraries_no_frameworks
      @uses_libraries_no_frameworks = begin
        accessors = file_accessors.reject { |a| a.spec.test_specification? }
        !accessors.empty? && accessors.all? do |file_accessor|
          !file_accessor.vendored_libraries.blank? && file_accessor.vendored_frameworks.blank?
        end
      end
    end

    def should_generate_module_map_for_a_pod_should_not_build?
      @should_generate_module_map_for_a_pod_should_not_build ||= begin
        self.uses_modular_headers? && contain_vendored_libraries_and_no_vendored_frameworks?
      end
    end

  end
end


module Pod
  class Installer
    class Xcode
      class PodsProjectGenerator
        class PodTargetInstaller


          FIX_MODULE_OLD_install = instance_method(:install!)
          define_method(:install!) do

              if (not target.should_build?) && target.should_generate_module_map_for_a_pod_should_not_build?
                  # special handling for pod with no source code

                  # Most code below are copied from original install! method
                  # Manually checked the 1.5.3/ 1.5.2/ 1.5.1/ 1.5.0, they are all the same. So it's safe.
                  UI.message "- Installing target `#{target.name}` #{target.platform}" do
                    add_target
                    create_support_files_dir

                    add_resources_bundle_targets
                    create_xcconfig_file

                    if target.defines_module?
                      create_module_map do |generator|
                        generator.headers.concat module_map_additional_headers
                      end
                      create_umbrella_header do |generator|
                        file_accessors = target.file_accessors
                        file_accessors = file_accessors.reject { |f| f.spec.test_specification? } if target.contains_test_specifications?
                        generator.imports += if header_mappings_dir
                                               file_accessors.flat_map(&:public_headers).map do |pathname|
                                                 pathname.relative_path_from(header_mappings_dir)
                                               end
                                             else
                                               file_accessors.flat_map(&:public_headers).map(&:basename)
                                             end
                      end
                    end

                    if target.requires_frameworks?
                      unless target.static_framework?
                        create_info_plist_file(target.info_plist_path, native_target, target.version, target.platform)
                      end
                      create_build_phase_to_symlink_header_folders
                    elsif target.uses_swift?
                      add_swift_static_library_compatibility_header_phase
                    end

                  end

              else
                  # call original
                  FIX_MODULE_OLD_install.bind(self).()
              end

          end # patch ended


        end
      end
    end
  end
end





module Pod
  class Installer
    class Xcode
      class PodsProjectGenerator


            # override
            #
            # Manually checked the 1.5.3/ 1.5.2/ 1.5.1/ 1.5.0, they are all the same. So it's safe.
            #
            def install_libraries
              UI.message '- Installing targets' do
                umbrella_headers_by_dir = pod_targets.map do |pod_target|
                  ## ---- modified
                  # next unless pod_target.should_build? && pod_target.defines_module?
                  next unless pod_target.defines_module?
                  next unless (pod_target.should_build? || pod_target.should_generate_module_map_for_a_pod_should_not_build? )
                  ## ---- modifed end
                  #
                  # contentx below is copied from original method

                  pod_target.umbrella_header_path
                end.compact.group_by(&:dirname)

                pod_targets.sort_by(&:name).each do |pod_target|
                  target_installer = PodTargetInstaller.new(sandbox, pod_target)
                  target_installer.umbrella_headers_by_dir = umbrella_headers_by_dir
                  target_installer.install!
                end

                aggregate_targets.sort_by(&:name).each do |target|
                  target_installer = AggregateTargetInstaller.new(sandbox, target)
                  target_installer.install!
                end

                add_system_framework_dependencies
              end
            end

      end
    end
  end
end



module Pod
  module Generator
    module XCConfig
      module XCConfigHelper

              # override
              #
              # Manually checked the 1.5.3/ 1.5.2/ 1.5.1, they are all the same. So it's safe.
              # 1.5.0 is slightly different at Line 147 (`dependent_targets.uniq.combination(2) do |a, b|`)
              # with no `uniq`, but I think it's not a problem.
              #
              def self.search_paths_for_dependent_targets(target, dependent_targets, test_xcconfig = false)
                ## ---- modified
                # dependent_targets = dependent_targets.select(&:should_build?)
                dependent_targets = dependent_targets.select { |target| target.should_build? || target.should_generate_module_map_for_a_pod_should_not_build? }
                ## ---- modifed end
                #
                # contentx below is copied from original method

                # Filter out dependent targets that are subsets of another target.
                subset_targets = []
                dependent_targets.uniq.combination(2) do |a, b|
                  if (a.specs - b.specs).empty?
                    subset_targets << a
                  elsif (b.specs - a.specs).empty?
                    subset_targets << b
                  end
                end
                dependent_targets -= subset_targets

                # Alias build dirs to avoid recursive definitions for pod targets and depending
                # on build settings which could be overwritten in the user target.
                build_settings = {
                    BUILD_DIR_VARIABLE[2..-2] => '${BUILD_DIR}',
                    CONFIGURATION_BUILD_DIR_VARIABLE[2..-2] => "#{BUILD_DIR_VARIABLE}/$(CONFIGURATION)$(EFFECTIVE_PLATFORM_NAME)",
                }

                # Scope pod targets as long as they are not test targets.
                if !test_xcconfig && target.respond_to?(:configuration_build_dir)
                  build_settings['CONFIGURATION_BUILD_DIR'] = target.configuration_build_dir(CONFIGURATION_BUILD_DIR_VARIABLE)
                end

                module_map_files = []
                unless dependent_targets.empty?
                  framework_search_paths = []
                  library_search_paths = []
                  swift_import_paths = []
                  dependent_targets.each do |dependent_target|
                    if dependent_target.requires_frameworks?
                      framework_search_paths << dependent_target.configuration_build_dir(CONFIGURATION_BUILD_DIR_VARIABLE)
                    else
                      ### -------------- modified ---------------
                      # library_search_paths << dependent_target.configuration_build_dir(CONFIGURATION_BUILD_DIR_VARIABLE)
                      # --- to ----
                      if dependent_target.should_build?
                        library_search_paths << dependent_target.configuration_build_dir(CONFIGURATION_BUILD_DIR_VARIABLE)
                      end
                      ### -------------- \modified ---------------
                      if dependent_target.defines_module?
                        module_map_file = if dependent_target.uses_swift?
                                            # for swift, we have a custom build phase that copies in the module map, appending the .Swift module
                                            "${PODS_CONFIGURATION_BUILD_DIR}/#{dependent_target.label}/#{dependent_target.product_module_name}.modulemap"
                                          else
                                            "${PODS_ROOT}/#{dependent_target.module_map_path.relative_path_from(dependent_target.sandbox.root)}"
                                          end
                        module_map_files << %(-fmodule-map-file="#{module_map_file}")
                        swift_import_paths << dependent_target.configuration_build_dir(CONFIGURATION_BUILD_DIR_VARIABLE) if dependent_target.uses_swift?
                      end
                    end
                  end
                  build_settings['FRAMEWORK_SEARCH_PATHS'] = XCConfigHelper.quote(framework_search_paths.uniq)
                  build_settings['LIBRARY_SEARCH_PATHS']   = XCConfigHelper.quote(library_search_paths.uniq)
                  build_settings['SWIFT_INCLUDE_PATHS']    = XCConfigHelper.quote(swift_import_paths.uniq)
                end

                other_swift_flags = module_map_files.tap(&:uniq!).flat_map { |f| ['-Xcc', f] }
                if target.is_a?(PodTarget) && !target.requires_frameworks? && target.defines_module? && !test_xcconfig
                  # make it possible for a mixed swift/objc static library to be able to import the objc from within swift
                  other_swift_flags += ['-import-underlying-module', '-Xcc', '-fmodule-map-file="${SRCROOT}/${MODULEMAP_FILE}"']
                end
                # unconditionally set these, because of (the possibility of) having to add the pod targets own module map file
                build_settings['OTHER_CFLAGS']           = module_map_files.join(' ')
                build_settings['OTHER_SWIFT_FLAGS']      = other_swift_flags.join(' ')

                build_settings
              end
      end
    end
  end
end

# ------ \fix for cocoapods 1.5 ------- 
}  # the root level string
